#!/usr/bin/perl

use strict;
use warnings;
use Getopt::Long qw(:config no_ignore_case bundling);
use Net::LDAP;
use Net::LDAPS;
use File::Basename qw(basename);
use List::Util qw (max);

#Constants
my $LOGFILE="/var/opt/novell/log/iprint/zen7policy_migration.log";
my $PORT = 636;
#Globals
my (%opts,$logfile,$i);
my ($host, $passwd, $basedn, $admin, $keepdefault, $ldap, $r, $removeold);
my $vlevel = 1;		#Default verbose level 1.
my $searchstring="objectclass=*printpolicy";
my $attnames=["zenwmiPrintPrinterList"];
my (@originalprinters, @migprinters);

#Main program
openLogfile();
parseOpts();
connectLdap();
modifyZenpolicy();
closeLdap();
#End Main program

#Function to open log file
sub openLogfile
{	
	if(open $logfile,">>$LOGFILE" )
	{	my $t = localtime(time);
		displayLine();
		printError("T:","$t.",0,0);
		displayLine();
	}
	else
	{	print "Error : Can not open log file $LOGFILE\n";
		exit(0);
	}
}
#Function to read parameters
sub parseOpts
{	my %opts;
        eval
        {
        	GetOptions("h|help" => \$opts{'help'},
                           "v|verbose+" => \$opts{'verbose'},
                           "s|host=s" => \$opts{'host'},
                           "a|admin=s" => \$opts{'admin'},
                           "p|passwd=s" => \$opts{'passwd'},
                           "b|basedn=s" => \$opts{'basedn'},
                           "d|keepdefault" => \$opts{'keepdefault'},
                           "r|removeold" => \$opts{'removeold'},
                           "l|linux" => \$opts{'linux'},
                           "n|netware" => \$opts{'netware'},
                           "f|file=s" => \$opts{'file'}) or usage();
	};
	usage("Options are not parsed: $@.") if $@;
	usage() if $opts{'help'};
	$vlevel = defined($opts{'verbose'}) ? $opts{'verbose'} : 1;
	#Validate parameters
	my $errmsg = "";
	$errmsg = "$errmsg\nPlease specify eDirectory replica server ip address." if !$opts{'host'};
	$errmsg = "$errmsg\nPlease specify fdn for Administrator user." if !$opts{'admin'};
	$errmsg = "$errmsg\nPlease specify password for Administrator user." if !$opts{'passwd'};
	$errmsg = "$errmsg\nPlease specify DN of container to search Zen7 print policy objects." if !$opts{'basedn'};
	my $suffix = defined($opts{'linux'}) ? "_L" : ( defined($opts{'netware'}) ? "_NW" : "");
	$errmsg = "$errmsg\nPlease specify Source OS." if($suffix eq "");
	$errmsg = "$errmsg\nPlease specify data file containing migrated printer names." if !$opts{'file'};
	printError("U:","$errmsg",1,1) if $errmsg;	
	
	$host = $opts{'host'};
        $passwd = $opts{'passwd'};
        $basedn = $opts{'basedn'};
        $admin = $opts{'admin'};
	$keepdefault = 0;
        $keepdefault = defined($opts{'keepdefault'}) ? $opts{'keepdefault'} : 0;
	$removeold = 0;
        $removeold = defined($opts{'removeold'}) ? $opts{'removeold'} : 0;
	#Read file for printer names.
	my ($IN, $PRINTLIST,$j);
	printError("E:","Can not open data file $opts{'file'}",0,1) if(!open IN,"<$opts{'file'}");
	$j = 0;
	while(<IN>)
       	{
               	chomp($_);
               	$originalprinters[$j] = $_;
       	        #Create migrated printer names from original printer names
                $migprinters[$j] = $_;
                $migprinters[$j] =~ s/$suffix,/,/;
       	        $j = $j + 1;
        }
       	close(IN);
	printError("E:","Data file $opts{'file'} does not contain any printer name.",0,1) if !$j;
}

#Display help
sub usage(;$)
{
	my $err = 0;
        if (@_)
        {
                my $warning = shift;
                print {*STDERR} "$warning\n" if $warning;
                $err = 1;
        }
	my $prgname = basename($0);
        print {$err ? *STDERR : *STDOUT} <<EOF;
Usage : $prgname -s <Host name or ip address> -a <Administrator FDN> -p <Administrator password> -b <Base dn> -f <File name containing migrated printer names>
Options:
-h | --help    				Print this summary.
-v | --verbose          		Output level(-v : shell, -v -v : log file, -v -v -v : both).
-s | --host <Host name or ip address>  	eDirectory replica server ip address.
-a | --admin <Administrator FDN>        Administrator FDN (e.g. cn=admin,o=novell).
-p | --passwd <Administrator password>  Administrator password. 
-b | --base-dn <DN>  			DN of container to search Zen7 print policy objects (e.g o=novell).
-d | --keepdefault 			Do not change default printer in Zen7 print policy.
-r | --removeold 			Remove old printer names from Zen7 print policy.
-l | --linux 				Migration Source OS is Linux.
-n | --netware 				Migration Source OS is Netware.
-f | --file <File name> 		File containing migrated printer names.
EOF
exit($err);
}

#Connect to the ldap server
sub connectLdap
{       printError("I:","Connecting to ldap server",0,0);
        my $count = 0;
        until($ldap = Net::LDAPS->new($host, port => $PORT))
        {
                printError("E:","Can not connect to ldaps://$host:$PORT/.",0,1) if ++$count > 10;
                sleep 1;
        }
        $r = $ldap->bind($admin, password => "$passwd", version=>3);
        printError("E:","Connection failed to ldaps://$host:$PORT/ : $r->error.",0,1) if $r->code;

        $r = $ldap->search(     base    =>      $basedn,
                                scope   =>      'sub',
                                filter  =>      $searchstring,
                                attrs   =>      $attnames);
        printError("E:","Disconnected : $r->error",0,1) if $r->code;
        printError("I:","Connection successful.",0,0);
}

#Close connection with ldap server
sub closeLdap
{       $ldap->unbind;
        printError("I:","Connection with ldap server closed",0,0);
}

#Modify Zen policy
sub modifyZenpolicy
{
	my $modifyFlag = 0;
	foreach my $entry ($r->entries)
	{	
		$modifyFlag = 1;
		my $objectname = ${$entry->{'asn'}}{'objectName'};
        	printError("I:","Object = $objectname",0,0);
                my (@zenprinters,$zenprinter,@zenprinters_no,$originalprinter);
                $i = 0;
		printError("I:","Current printer list of Zen7 print policy : ",0,0);
		foreach $zenprinter (@{${$entry->{'asn'}}{'attributes'}[0]->{'vals'}})
		{	$zenprinter =~ s/#/ /g;
                        ($zenprinters[$i],$zenprinters_no[$i]) = split(/ /,$zenprinter);
			printError(" :","$zenprinters[$i]",0,0);
                        $i = $i + 1;
                }
		my $j = 0;
		my $l = $i;
		my ($m,$n);
		my $changed = 0;
		my $nooforiginalprinters = @zenprinters;
		my $max_index = max(@zenprinters_no) + 1;
                foreach $originalprinter (@originalprinters)
		{	
			for( $n = 0; $n < $nooforiginalprinters; $n = $n + 1 )
                        {        
				#if($zenprinters[$n] eq $originalprinter)
				if($zenprinters[$n] =~ m/^$originalprinter$/i)
				{
					my @ser = grep(/^$migprinters[$j]$/,@zenprinters);
                                	if( @ser == 0)
                                	{       
						$changed = 1;
						if($removeold)
						{
							$zenprinters[$n] = $migprinters[$j];
						}
						else
						{
							#create entry in zen policy object of eDirectory
                                        		if( $keepdefault == 0 && $zenprinters_no[$n] == 0 )
	                                        	{
        	                                        	for( $m = 0; $m < $l; $m = $m + 1 )
                	                                	{        $zenprinters_no[$m] = $zenprinters_no[$m] + 1;
                        	                        	}
                                	                	$zenprinters_no[$l] = 0;
								printError("I:","Old default printer : $zenprinters[$n]",0,0);
                                        	        	printError("I:","New default printer : $migprinters[$j]",0,0);
	                                        	}
        	                                	else
                	                        	{       $zenprinters_no[$l] = $max_index;
                        	                	}
                                	        	$zenprinters[$l] = $migprinters[$j];
                                        		$max_index = $max_index + 1;
	                                        	$l = $l + 1;
        	                        	}
					}
				}
                        }
			$j = $j + 1;
                }
		#Update zen policy object of eDirectory
		if( $changed )
                {       #Atleast one entry is newly added
                        #Upate eDirectory object
                        my @zenPrinterlist;
                        printError("I:","Updated printer list of Zen7 print policy :",0,0);
                        for( $i = 0; $i < $l; $i = $i + 1 )
	                {       printError(" :","$zenprinters[$i].",0,0);
                                $zenPrinterlist[$i] = join('#',$zenprinters[$i],$zenprinters_no[$i]);
                        }
                        $entry->replace('zenwmiPrintPrinterList' => ["$zenPrinterlist[0]#"]);
                        for( $i = 1; $i < $l; $i = $i + 1 )
                        {       $entry->add('zenwmiPrintPrinterList' => ["$zenPrinterlist[$i]#"]);
                        }
	                my $error_msg = $entry->update($ldap);
                        if( $error_msg->{resultCode} != 0 )
                        {       printError("E:","$objectname :: $error_msg->{resultCode}.",0,0);
                                printError("E:"," : $error_msg->{erroMessage}",0,0) if $error_msg->{erroMessage};
                                print "$objectname :: $error_msg->{resultCode} : $error_msg->{erroMessage}.\n";
                        }
                        else
                        {       printError("S:","$objectname :: Policy is modified successfully.",0,0);
                                print "$objectname :: Policy is modified successfully.\n";
                        }
		}
		displayLine();
        }
	if($modifyFlag == 0)
	{
		printError("I:","No Zen7 print policy in context $basedn",0,0);	
        }
}
#Display line on console
sub displayLine
{	printError("","______________________________________",0,0);
}
#Display error message
sub printError($)
{       my $heading = shift;
        my $warning = shift;
        my $usage = shift;
        my $exitflag = shift;
        print {*STDERR} "$heading$warning\n" if($vlevel == 1 || $vlevel >= 3);
        print {*$logfile} "$heading$warning\n" if($vlevel == 2 || $vlevel >= 3);
        usage() if $usage;
        exit(0) if $exitflag;
}
